/*
   main.c 
   Proiectarea cu microprocesoare 2013
   */

#define F_CPU 16000000UL
#include <util/delay.h>
#include <avr/interrupt.h>
#include <math.h>

#include "T6963c.h"
#include "game.h"


#define XCHANNEL 3
#define YCHANNEL 4


/* TODO 2: Setati valorile aici pentru a reflecta valorile acceleratiilor
 * 	pentru placa pusa orizontal pe masa */
#define ACCX_0G 317
#define ACCY_0G 324


#define THRESHOLD 15
#define SPEED_INCREMENT 0.005f


int ADC_get(uint8_t channel)
{
	/* TODO 1: ADC get pe canalul channel */
	return ADC;
}

void ADC_init(void)
{
	/* TODO 1: ADC init */
}

void print_accel(int xval, int yval)
{
	static uint8_t first_time = 1;
	uint8_t i = 0;

	if (first_time)
	{
		T6963cPutStringXY_P(0, 0, PSTR("Accel.X"));
		T6963cPutStringXY_P(0, 1, PSTR("Accel.Y"));
		first_time = 0;
	}
	T6963cSetCursorXY(8, i);


	/* TODO 2: Afisati cu T6963cPutStringXY sau cu 
  		T6963cPutCharXY valorile acceleratiilor pe X si Y */

	
}


int main(void)
{
	DDRC = 0xFF; 
	PORTC = 0xFF;
	_delay_ms(100);


	//XOR mode, internal CG, Text and Graphic, Blinking Cursor	
	T6963cInit(	T6963C_MODE_XOR | T6963C_CG_INTERNALROM, T6963C_TEXT_GRAPHIC  );

	ADC_init();

	/* Initialize game */
	init_game();

	while(1)
	{	
		/* Erase old moving components */
		//erase ball
		draw_ball(x,y, 0); 
		//erase pad
		draw_pad(x_pad, y_pad, 0);

		/* Read and display ADC conversion results on X and Y axis*/
		float oldx = x;
		float oldy = y;
		int xval = 0 , yval = 0;

		xval = ADC_get(XCHANNEL);	
		yval = ADC_get(YCHANNEL);	
		print_accel(xval, yval);


		/******* Game Logic *******/


		/* Check for win/loss condition */
		if (win > 0) 
		{
			T6963cPutStringXY_P(0, 6, PSTR("YOU WIN"));
			_delay_ms(1000);
			init_game();
		}
		else if (win < 0)
		{
			T6963cPutStringXY_P(0, 6, PSTR("YOU LOSE"));
			_delay_ms(1000);
			init_game();
		}
		else
			T6963cPutStringXY_P(0, 6, PSTR("        "));	


		/* calculate the new speed and coordinates */
		if (xval - ACCX_0G > THRESHOLD)
			x_pad -= PAD_INCREMENT;
		else if (ACCX_0G - xval > THRESHOLD)
			x_pad += PAD_INCREMENT;

		// put pad in limits of playing area
		if (x_pad < X1_LIMIT + 1) x_pad = X1_LIMIT + 1;
		if (x_pad > X2_LIMIT - PAD_LENGTH) x_pad = X2_LIMIT - PAD_LENGTH;

		// modify positions according to speed
		x += xspeed;
		y += yspeed;

		// check if ball hit pad
		int result;
		if ((result = check_pad(x,y,x_pad, y_pad)) >= 0)
		{
			PORTC = 0;
			yspeed *= -1;
			float angle = (result * M_PI/2)/ PAD_LENGTH - M_PI/4;

			float xs, ys;

			xs = xspeed * cos(angle) - yspeed * sin(angle);
			ys = xspeed * sin(angle) + yspeed * cos(angle);

			xspeed = xs;
			yspeed = ys;
		}

		// check margins -> bounce
		switch(check_margins(x,y))
		{
			case HIT_VERT:
				x = oldx;
				y = oldy;
				yspeed *= -1;
				break;
			case HIT_HORIZ:
				x = oldx;
				y = oldy;
				xspeed *= -1;
				break;
			case HIT_LOST:
				win = -1;
				break;

		}

		// check bricks -> don't bounce
		check_bricks(x,y);

		if (remaining_bricks == 0)
			win = 1;


		draw_ball(x,y, 1);
		draw_pad(x_pad, y_pad, 1);

		_delay_ms(25);
	}


	return 0;
}
